<?php

/**
 * @file
 * Contains \Drupal\publish_away\Form\PublishAwaySettingsForm.
 */
 
namespace Drupal\publish_away\Form;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\publish_away\Controller\Facebook;
use Drupal\publish_away\Controller\TmhOAuth;
use Drupal\publish_away\Controller\TmhUtilities;
 
class PublishAwaySettingsForm extends ConfigFormBase {
  
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'system_publish_away_settings';
  }
  
  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, array &$form_state) {
    $publish_away_config = $this->configFactory->get('publish_away.settings');
    $form['#tree'] = TRUE;
    
    // Facebook settings.
    $form['facebook'] = array(
      '#type' => 'details',
      '#title' => $this->t('Facebook settings'),
      '#collapsible' => TRUE,
    );
    $form['facebook']['html'] = array(
      '#type' => 'markup',
      '#markup' => $this->facebook_login(),
    );
    $form['facebook']['appId'] = array(
      '#title' => $this->t('AppId'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('facebook.appId'),
      '#required' => FALSE,
    );
    $form['facebook']['secret'] = array(
      '#title' => $this->t('Secret key'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('facebook.secret'),
      '#required' => FALSE,
    );
    $form['facebook']['client_token'] = array(
      '#title' => $this->t('Client Token'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('facebook.client_token'),
      '#required' => FALSE,
    );
    $form['facebook']['page_id'] = array(
      '#title' => $this->t('Facebook Page ID'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('facebook.page_id'),
      '#required' => FALSE,
    );
    
    // Twitter settings.
    $form['twitter'] = array(
      '#type' => 'details',
      '#title' => $this->t('Twitter settings'),
      '#collapsible' => TRUE,
    );
    $form['twitter']['html'] = array(
      '#type' => 'markup',
      '#markup' => $this->twitter_login(),
    );
    $form['twitter']['consumer_key'] = array(
      '#title' => $this->t('Consumer key'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('twitter.consumer_key'),
      '#required' => FALSE,
    );
    $form['twitter']['consumer_secret'] = array(
      '#title' => $this->t('Consumer secret key'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('twitter.consumer_secret'),
      '#required' => FALSE,
    );
    $form['twitter']['oauth_token'] = array(
      '#title' => $this->t('Access token'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('twitter.oauth_token'),
      '#required' => FALSE,
    );
    $form['twitter']['oauth_token_secret'] = array(
      '#title' => $this->t('Access token secret'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('twitter.oauth_token_secret'),
      '#required' => FALSE,
    );
    // Bitly settings.
    $form['bitly'] = array(
      '#type' => 'details',
      '#title' => $this->t('Bitly settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#description' => $this->t('To shorten the links visit <a href="@bitlyurl" target="_blank" />@bitlyurl</a> and get your Bilty API Key', array('@bitlyurl' => 'https://bitly.com/a/your_api_key')),
    );
    $form['bitly']['username'] = array(
      '#title' => $this->t('Bitly username'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('bitly.username'),
      '#required' => FALSE,
    );
    $form['bitly']['apikey'] = array(
      '#title' => $this->t('Bitly API Key'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('bitly.apikey'),
      '#required' => FALSE,
    );

    return parent::buildForm($form, $form_state);
  }
  
  /**
   * Implements \Drupal\Core\Form\FormInterface::validateForm().
   */
  public function validateForm(array &$form, array &$form_state) {
    form_state_values_clean($form_state);
    $publish_away_settings = $form_state['values'];
    if (!empty($publish_away_settings['facebook'])) {
      if (trim($publish_away_settings['facebook']['appId']) == '' && trim($publish_away_settings['facebook']['secret']) != '') {
        $this->setFormError('facebook][appId', $form_state, $this->t('Facebook appId is missing.'));
      }
      if (trim($publish_away_settings['facebook']['appId']) != '' && trim($publish_away_settings['facebook']['secret']) == '') {
        $this->setFormError('facebook][secret', $form_state, $this->t('Facebook secret key is missing.'));
      }
    }
    if (!empty($publish_away_settings['twitter'])) {
      if (trim($publish_away_settings['twitter']['consumer_key']) != '' && trim($publish_away_settings['twitter']['consumer_secret']) == '') {
        $this->setFormError('twitter][consumer_secret', $form_state, $this->t('Twitter consumer secret key is missing.'));
      }
      if (trim($publish_away_settings['twitter']['consumer_key']) == '' && trim($publish_away_settings['twitter']['consumer_secret']) != '') {
        $this->setFormError('twitter][consumer_key', $form_state, $this->t('Twitter consumer key is missing.'));
      }
      if (trim($publish_away_settings['twitter']['consumer_key']) != '' && trim($publish_away_settings['twitter']['oauth_token']) == '') {
        $this->setFormError('twitter][oauth_token', $form_state, $this->t('Twitter access token is missing.'));
      }
      if (trim($publish_away_settings['twitter']['consumer_key']) != '' && trim($publish_away_settings['twitter']['oauth_token_secret']) == '') {
        $this->setFormError('twitter][oauth_token_secret', $form_state, $this->t('Twitter access token secret is missing.'));
      }
    }
    if (!empty($publish_away_settings['bitly'])) {
      if (trim($publish_away_settings['bitly']['username']) == '' && trim($publish_away_settings['bitly']['apikey']) != '') {
        $this->setFormError('bitly][username', $form_state, $this->t('Bitly username is missing.'));
      }
      if (trim($publish_away_settings['bitly']['username']) != '' && trim($publish_away_settings['bitly']['apikey']) == '') {
        $this->setFormError('bitly][apikey', $form_state, $this->t('Bitly API Key is missing.'));
      }
    }
  }
  
  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, array &$form_state) {
    $this->configFactory->get('publish_away.settings')
      ->set('facebook.appId', $form_state['values']['facebook']['appId'])
      ->set('facebook.secret', $form_state['values']['facebook']['secret'])
      ->set('facebook.client_token', $form_state['values']['facebook']['client_token'])
      ->set('facebook.page_id', $form_state['values']['facebook']['page_id'])
      ->set('twitter.consumer_key', $form_state['values']['twitter']['consumer_key'])
      ->set('twitter.consumer_secret', $form_state['values']['twitter']['consumer_secret'])
      ->set('twitter.oauth_token', $form_state['values']['twitter']['oauth_token'])
      ->set('twitter.oauth_token_secret', $form_state['values']['twitter']['oauth_token_secret'])
      ->set('bitly.username', $form_state['values']['bitly']['username'])
      ->set('bitly.apikey', $form_state['values']['bitly']['apikey'])
      ->save();
    
    parent::submitForm($form, $form_state);
  }
  
  private function facebook_login() {
    $publish_away_config = $this->configFactory->get('publish_away.settings');
    // Facebook connect.
    $facebook_html = '';
    $config = array(
      'appId'  => $publish_away_config->get('facebook.appId'),
      'secret' => $publish_away_config->get('facebook.secret'),   
    );
    $facebook = new Facebook($config);
    $user_id = $facebook->getUser();
    if (intval($user_id) > 0) {
      // We have a user ID, so probably a logged in user.
      // If not, we'll get an exception, which we handle below.
      try {
        $user_profile = $facebook->api('/me','GET');
        $token = $facebook->getAccessToken();

        $this->configFactory->get('publish_away.settings')
          ->set('facebook.user_profile', $user_profile)
          ->set('facebook.id', $user_profile['id'])
          ->set('facebook.name', $user_profile['name'])
          ->set('facebook.token', $token)
          ->save();
      }
      catch(FacebookApiException $e) {
        // If the user is logged out, you can have a 
        // user ID even though the access token is invalid.
        // In this case, we'll get an exception, so we'll
        // just ask the user to login again here.
        $login_url = $facebook->getLoginUrl();
        $facebook_html .= l($this->t('Please you need to connect to the Facebook account which will display selected contents'), $login_url);
        error_log($e->getType());
        error_log($e->getMessage());
      }
      $fb_profile = $publish_away_config->get('facebook.user_profile');
      $facebook_html .= '<strong>' . $this->t('Facebook username') . ' :</strong> ' . $fb_profile['username'];
      $facebook_html .= '<br /><strong>' . $this->t('Facebook name') . ' :</strong> ' . $publish_away_config->get('facebook.name');
      $facebook_html .= '<br /><strong>' . $this->t('Location') . ' :</strong> ' . $fb_profile['location']['name'];
      if (isset($fb_profile['quotes']) && trim($fb_profile['quotes']) != '') {
        $facebook_html .= '<br /><strong>' . $this->t('Last Facebook status') . ' :</strong> ' . $fb_profile['quotes'];
      }
    }
    else {
      // No user, print a link for the user to login.
      $login_url = $facebook->getLoginUrl();
      $facebook_html .= l($this->t('Please you need to connect to the Facebook account which will display selected contents'), $login_url);
      $this->configFactory->get('publish_away.settings')
        // ->set('facebook.appId', '')
        ->set('facebook.token', '')
        ->save();
    }
    return $facebook_html;
  }

  private function twitter_login() {
    $publish_away_config = $this->configFactory->get('publish_away.settings');
    $twitter_html = '';
    $consumer_key = $publish_away_config->get('twitter.consumer_key');
    $consumer_secret = $publish_away_config->get('twitter.consumer_secret');
    $oauth_token = $publish_away_config->get('twitter.oauth_token');
    $oauth_token_secret = $publish_away_config->get('twitter.oauth_token_secret');

    if ($consumer_key != '' && $consumer_secret != '' && $oauth_token != '' && $oauth_token_secret != '') {
  
      $TmhOAuth = new TmhOAuth(array(
       'consumer_key' => $consumer_key,
        'consumer_secret' => $consumer_secret,
        'user_token' => $oauth_token,
        'user_secret' => $oauth_token_secret,
      ));
      
      $response_code = $TmhOAuth->request('GET', $TmhOAuth->url('1.1/account/verify_credentials'));
      if ($response_code == 200) {
        $twitter_user = json_decode($TmhOAuth->response['response']);
        if (isset($twitter_user->id) && trim($twitter_user->id) != '') {
          $twitter_html .= '<strong>' . t('Twitter account') . ' :</strong> ' . $twitter_user->name;
          if (isset($twitter_user->status) && isset($twitter_user->status->text) && trim($twitter_user->status->text) != '') {
            $twitter_html .= '<br /><strong>' . t('Your last Tweet') . ' :</strong> ' . $twitter_user->status->text;
          }
          $twitter_html .= '<br /><img style="veryical-align:middle;" src="'. $twitter_user->profile_image_url .'" alt="'. t('@user image', array('@user' => $twitter_user->name)) .'" />';
        }
      }
      else {
        $twitter_html = $this->t('Twitter connection failed.');
      }
    }
    else {
      $twitter_html = $this->t('Twitter connection failed.');
    }
    return $twitter_html;
  }

  public function facebook_logout() {
    $publish_away_config = $this->configFactory->get('publish_away.settings');
    // Log out the Facebook acount.
    $config = array(
      'appId'  => $publish_away_config->get('facebook.appId'),
      'secret' => $publish_away_config->get('facebook.secret'),   
    );
    $facebook = new Facebook($config);
    $this->configFactory->get('publish_away.settings')
      ->set('facebook.token', '')
      ->set('facebook.id', '')
      ->set('facebook.name', '')
      ->set('facebook.user_profile', '')
      ->save();
    $facebook->api('/' . $publish_away_config->get('facebook.id') . '/permissions', 'DELETE');
    $form = \Drupal::formBuilder()->getForm($this->getFormId());
    return $form;
  }
}
