<?php

/**
 * @file
 * Contains \Drupal\publish_away\Plugin\Field\FieldType\PublishAwayFacebook.
 */

namespace Drupal\publish_away\Plugin\Field\FieldType;

use Drupal\Core\Field\ConfigFieldItemBase;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * Plugin implementation of the 'publish_away' field type.
 *
 * @FieldType(
 *   id = "publish_away_facebook",
 *   label = @Translation("Publish away (Facebook)"),
 *   description = @Translation("This field stores a Facebook message in the database as a string."),
 *   default_widget = "publish_away_facebook_widget",
 *   default_formatter = "publish_away_default_formatter",
 * )
 */
class PublishAwayFacebook extends ConfigFieldItemBase {

  /**
   * Definitions of the contained properties.
   *
   * @var array
   */
  static $propertyDefinitions;

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldDefinitionInterface $field_definition) {
    return array(
      'columns' => array(
        'enabled' => array(
          'type' => 'int',
          'default' => 0,  
          'not null' => FALSE,
        ),
        'message' => array(
          'type' => 'text',
          'size' => 'big',
          'not null' => FALSE,
        ),
        'file_fid' => array(
          'type' => 'int',
          'not null' => FALSE,
        ),
      ),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getPropertyDefinitions() {
    if (!isset(static::$propertyDefinitions)) {
      static::$propertyDefinitions['enabled'] = DataDefinition::create('boolean')
        ->setLabel(t('Enable sending'));
      static::$propertyDefinitions['message'] = DataDefinition::create('string')
        ->setLabel(t('Message'));
    }
    return static::$propertyDefinitions;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $enabled = $this->get('enabled')->getValue();
    $message = $this->get('message')->getValue();
    return (int) $enabled && (trim($message) === '' || $message === NULL);
  }

  /**
   * {@inheritdoc}
   */
  public function getConstraints() {
    $constraint_manager = \Drupal::typedDataManager()->getValidationConstraintManager();
    $constraints = parent::getConstraints();

    $max_length = 115;
    $constraints[] = $constraint_manager->create('ComplexData', array(
      'message' => array(
        'Length' => array(
          'max' => $max_length,
          'maxMessage' => t('%name: may not be longer than @max characters.', array('%name' => $this->getFieldDefinition()->getLabel(), '@max' => $max_length)),
        )
      ),
    ));

    return $constraints;
  }

}
